@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -20,8 +20,9 @@
 #include "TransformationManager.h"
 
 static TransformationManager *TransMgr;
+static int ErrorCode = -1;
 
-static void PrintVersion(void)
+static void PrintVersion()
 {
   llvm::outs() << "clang_delta " << PACKAGE_VERSION << "\n";
 #ifdef GIT_VERSION
@@ -30,7 +31,7 @@ static void PrintVersion(void)
   // XXX print copyright, contact info, etc.?
 }
 
-static void PrintHelpMessage(void)
+static void PrintHelpMessage()
 {
   PrintVersion();
   llvm::outs() << "\n";
@@ -49,7 +50,8 @@ static void PrintHelpMessage(void)
   llvm::outs() << "print the program version number\n";
 
   llvm::outs() << "  --verbose-transformations: ";
-  llvm::outs() << "print verbose description messages for all transformations\n";
+  llvm::outs() << "print verbose description messages for all ";
+  llvm::outs() << "transformations\n";
 
   llvm::outs() << "  --transformation=<name>: ";
   llvm::outs() << "specify the transformation\n";
@@ -58,11 +60,19 @@ static void PrintHelpMessage(void)
   llvm::outs() << "print the names of all available transformations\n";
 
   llvm::outs() << "  --query-instances=<name>: ";
-  llvm::outs() << "query available transformation instances for a given transformation\n";
+  llvm::outs() << "query available transformation instances for a given ";
+  llvm::outs() << "transformation\n";
 
   llvm::outs() << "  --counter=<number>: ";
   llvm::outs() << "specify the instance of the transformation to perform\n";
 
+  llvm::outs() << "  --to-counter=<number>: ";
+  llvm::outs() << "specify the ending instance of the transformation to ";
+  llvm::outs() << "perform (when this option is given, clang_delta will ";
+  llvm::outs() << "rewrite multiple instances [counter,to-counter] ";
+  llvm::outs() << "simultaneously. Note that currently only ";
+  llvm::outs() << "replace-function-def-with-decl supports this feature.)\n";
+
   llvm::outs() << "  --output=<filename>: ";
   llvm::outs() << "specify where to output the transformed source code ";
   llvm::outs() << "(default: stdout)\n";
@@ -82,7 +92,7 @@ static void Die(const std::string &Message)
 {
   llvm::outs() << "Error: " << Message << "\n";
   TransformationManager::Finalize();
-  exit(-1);
+  exit(ErrorCode);
 }
 
 static void HandleOneArgValue(const std::string &ArgValueStr, size_t SepPos)
@@ -112,11 +122,24 @@ static void HandleOneArgValue(const std::string &ArgValueStr, size_t SepPos)
     int Val;
     std::stringstream TmpSS(ArgValue);
 
-    if (!(TmpSS >> Val))
-      DieOnBadCmdArg("--" + ArgValueStr);
+    if (!(TmpSS >> Val)) {
+      ErrorCode = TransformationManager::ErrorInvalidCounter;
+      Die("Invalid counter[" + ArgValueStr + "]");
+    }
 
     TransMgr->setTransformationCounter(Val);
   }
+  else if (!ArgName.compare("to-counter")) {
+    int Val;
+    std::stringstream TmpSS(ArgValue);
+
+    if (!(TmpSS >> Val)) {
+      ErrorCode = TransformationManager::ErrorInvalidCounter;
+      Die("Invalid to-counter[" + ArgValueStr + "]");
+    }
+
+    TransMgr->setToCounter(Val);
+  }
   else if (!ArgName.compare("output")) {
     TransMgr->setOutputFileName(ArgValue);
   }
@@ -179,13 +202,13 @@ int main(int argc, char **argv)
   }
 
   std::string ErrorMsg;
-  if (!TransMgr->verify(ErrorMsg))
+  if (!TransMgr->verify(ErrorMsg, ErrorCode))
     Die(ErrorMsg);
 
   if (!TransMgr->initializeCompilerInstance(ErrorMsg))
     Die(ErrorMsg);
 
-  if (!TransMgr->doTransformation(ErrorMsg)) {
+  if (!TransMgr->doTransformation(ErrorMsg, ErrorCode)) {
     // fail to do transformation
     Die(ErrorMsg);
   }
