@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013, 2015 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -17,6 +17,7 @@
 #include <algorithm>
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -85,6 +86,9 @@ const Type *CombLocalVarCollectionVisitor::getTypeFromDeclStmt(DeclStmt *DS)
 
 bool CombLocalVarCollectionVisitor::VisitCompoundStmt(CompoundStmt *CS)
 {
+  if (ConsumerInstance->isInIncludedFile(CS))
+    return true;
+
   ConsumerInstance->DeclStmts.clear();
 
   for (CompoundStmt::body_iterator I = CS->body_begin(), 
@@ -166,7 +170,6 @@ void CombineLocalVarDecl::doCombination(void)
 
 CombineLocalVarDecl::~CombineLocalVarDecl(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
+  delete CollectionVisitor;
 }
 
