@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2015 The University of Utah
+// Copyright (c) 2012, 2013, 2015, 2016 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -105,8 +105,8 @@ bool LocalToGlobalCollectionVisitor::VisitVarDecl(VarDecl *VD)
 {
   TransAssert(CurrentFuncDecl && "NULL CurrentFuncDecl!");
 
-  if (!VD->isLocalVarDecl() || VD->isStaticLocal() || 
-      VD->hasExternalStorage() || 
+  if (ConsumerInstance->isInIncludedFile(VD) || !VD->isLocalVarDecl() ||
+      VD->isStaticLocal() || VD->hasExternalStorage() ||
       ConsumerInstance->SkippedVars.count(VD->getCanonicalDecl()))
     return true;
 
