@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -13,6 +13,7 @@
 
 #include <string>
 #include "llvm/ADT/DenseMap.h"
+#include "llvm/ADT/SmallPtrSet.h"
 #include "Transformation.h"
 
 namespace clang {
@@ -52,6 +53,8 @@ private:
 
   llvm::DenseMap<const clang::VarDecl *, clang::DeclGroupRef> VarToDeclGroup;
 
+  llvm::SmallPtrSet<const clang::VarDecl *, 10> SkippedVars;
+
   RemoveUnusedVarAnalysisVisitor *AnalysisVisitor;
 
   clang::VarDecl *TheVarDecl;
