@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -104,7 +104,7 @@ private:
 
 bool RenameCXXMethodCollectionVisitor::VisitCXXRecordDecl(CXXRecordDecl *RD)
 {
-  if (!RD->hasDefinition())
+  if (ConsumerInstance->isInIncludedFile(RD) || !RD->hasDefinition())
     return true;
   const CXXRecordDecl *RDDef = RD->getDefinition();
   ConsumerInstance->handleOneCXXRecordDecl(RDDef);
@@ -113,6 +113,8 @@ bool RenameCXXMethodCollectionVisitor::VisitCXXRecordDecl(CXXRecordDecl *RD)
 
 bool RenameCXXMethodCollectionVisitor::VisitCXXMethodDecl(CXXMethodDecl *MD)
 {
+  if (ConsumerInstance->isInIncludedFile(MD))
+    return true;
   const CXXMethodDecl *CanonicalMD = MD->getCanonicalDecl();
   if(ConsumerInstance->NewMethodNames.find(CanonicalMD) != 
        ConsumerInstance->NewMethodNames.end())
@@ -188,7 +190,8 @@ bool RenameCXXMethodVisitor::TraverseClassTemplateDecl(ClassTemplateDecl *D)
 
 bool RenameCXXMethodVisitor::VisitCXXMethodDecl(CXXMethodDecl *MD)
 {
-  if (ConsumerInstance->isSpecialCXXMethod(MD))
+  if (ConsumerInstance->isSpecialCXXMethod(MD) ||
+      ConsumerInstance->isInIncludedFile(MD))
     return true;
 
   const CXXMethodDecl *CanonicalMD = MD->getCanonicalDecl();
