@@ -55,7 +55,8 @@ private:
 bool SimplifyNestedClassVisitor::VisitCXXRecordDecl(
        CXXRecordDecl *CXXRD)
 {
-  if (ConsumerInstance->isSpecialRecordDecl(CXXRD) || !CXXRD->hasDefinition())
+  if (ConsumerInstance->isInIncludedFile(CXXRD) ||
+      ConsumerInstance->isSpecialRecordDecl(CXXRD) || !CXXRD->hasDefinition())
     return true;
   ConsumerInstance->handleOneCXXRecordDecl(CXXRD->getDefinition());
   return true;
