@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -270,6 +270,12 @@ protected:
 
   unsigned getNumExplicitDecls(const clang::CXXRecordDecl *CXXRD);
 
+  bool isInIncludedFile(clang::SourceLocation Loc) const;
+
+  bool isInIncludedFile(const clang::Decl *D) const;
+
+  bool isInIncludedFile(const clang::Stmt *S) const;
+
   const std::string Name;
 
   int TransformationCounter;
