@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -129,11 +129,33 @@ bool TransformationManager::initializeCompilerInstance(std::string &ErrorMsg)
   }
 
   TargetOptions &TargetOpts = ClangInstance->getTargetOpts();
-  TargetOpts.Triple = LLVM_DEFAULT_TARGET_TRIPLE;
+
+  if (const char *env = getenv("CREDUCE_TARGET_TRIPLE")) {
+    TargetOpts.Triple = std::string(env);
+  } else {
+    TargetOpts.Triple = LLVM_DEFAULT_TARGET_TRIPLE;
+  }
+
   TargetInfo *Target = 
     TargetInfo::CreateTargetInfo(ClangInstance->getDiagnostics(),
                                  ClangInstance->getInvocation().TargetOpts);
   ClangInstance->setTarget(Target);
+
+  if (const char *env = getenv("CREDUCE_INCLUDE_PATH")) {
+    HeaderSearchOptions &HeaderSearchOpts = ClangInstance->getHeaderSearchOpts();
+
+    const std::size_t npos = std::string::npos;
+    std::string text = env;
+
+    std::size_t now = 0, next = 0;
+    do {
+      next = text.find(':', now);
+      std::size_t len = (next == npos) ? npos : (next - now);
+      HeaderSearchOpts.AddPath(text.substr(now, len), clang::frontend::Angled, false, false);
+      now = next + 1;
+    } while(next != npos);
+  }
+
   ClangInstance->createFileManager();
   ClangInstance->createSourceManager(ClangInstance->getFileManager());
   ClangInstance->createPreprocessor(TU_Complete);
@@ -147,7 +169,7 @@ bool TransformationManager::initializeCompilerInstance(std::string &ErrorMsg)
   ClangInstance->setASTConsumer(
     std::unique_ptr<ASTConsumer>(CurrentTransformationImpl));
   Preprocessor &PP = ClangInstance->getPreprocessor();
-  PP.getBuiltinInfo().InitializeBuiltins(PP.getIdentifierTable(),
+  PP.getBuiltinInfo().initializeBuiltins(PP.getIdentifierTable(),
                                          PP.getLangOpts());
 
   if (!ClangInstance->InitializeSourceManager(FrontendInputFile(SrcFileName, IK))) {
