@@ -34,6 +34,11 @@ sub advance ($$$) {
     return \$index;
 }
 
+sub advance_on_success ($$$) {
+    (my $cfile, my $arg, my $state) = @_;
+    return $state;
+}
+
 sub do_transform($$) {
     my ($cfile, $pattern) = @_;
 
