@@ -1,6 +1,6 @@
 ## -*- mode: Perl -*-
 ##
-## Copyright (c) 2012, 2013, 2015 The University of Utah
+## Copyright (c) 2012, 2013, 2015, 2016 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -20,7 +20,8 @@ my $astyle;
 my $clang_format;
 my $indent;
 
-my $indent_opts = "-nbad -nbap -nbbb -cs -pcs -prs -saf -sai -saw -sob -ss ";
+my $astyle_opts = "--options=none";
+my $indent_opts = "-npro -nbad -nbap -nbbb -cs -pcs -prs -saf -sai -saw -sob -ss ";
 
 sub check_prereqs () {
     $astyle =
@@ -30,7 +31,7 @@ sub check_prereqs () {
     $indent =
 	find_external_program(creduce_config::INDENT, "indent");
 
-    return defined ($astyle) && defined ($clang_format) && defined ($indent);
+    return defined ($clang_format);
 }
 
 sub new ($$) {
@@ -45,41 +46,65 @@ sub advance ($$$) {
     return \$index;
 }
 
+sub advance_on_success ($$$) {
+    (my $cfile, my $arg, my $state) = @_;
+    my $index = ${$state};
+    $index++;
+    return \$index;
+}
+
+sub invoke_indent ($) {
+    (my $cfile) = @_;
+    if ($^O eq "MSWin32") {
+	system qq{"$indent" $indent_opts $cfile > NUL 2>&1};
+    } else {
+	system qq{"$indent" $indent_opts $cfile >/dev/null 2>&1};
+    }
+}
+
+sub invoke_astyle ($) {
+    (my $cfile) = @_;
+    if ($^O eq "MSWin32") {
+	system qq{"$astyle" $astyle_opts $cfile > NUL 2>&1};
+    } else {
+	system qq{"$astyle" $astyle_opts $cfile >/dev/null 2>&1};
+    }
+}
+
+sub invoke_clang_format ($) {
+    (my $cfile) = @_;
+    if ($^O eq "MSWin32") {
+	system qq{"$clang_format" -i $cfile > NUL 2>&1};
+    } else {
+	system qq{"$clang_format" -i $cfile >/dev/null 2>&1};
+    }
+}
+
 sub transform ($$$) {
     (my $cfile, my $arg, my $state) = @_;
     my $index = ${$state};
-    if (0) {
-    } elsif ($arg eq "regular") {
+    my $old = read_file($cfile);
+  AGAIN:
+    if ($arg eq "regular") {
 	return ($STOP, \$index) unless ($index == 0);
-    if ($^O eq "MSWin32") {
-    system qq{"$indent" $indent_opts $cfile > NUL 2>&1};
-    } else {
-    system qq{"$indent" $indent_opts $cfile >/dev/null 2>&1};
-    }
+	invoke_clang_format($cfile);
     } elsif ($arg eq "final") {
 	if ($index == 0) {
-        if ($^O eq "MSWin32") {
-	    system qq{"$indent" $cfile > NUL 2>&1};
-        } else {
-	    system qq{"$indent" $cfile >/dev/null 2>&1};
-        }
+	    invoke_indent($cfile) if defined ($indent);
 	} elsif ($index == 1) {
-        if ($^O eq "MSWin32") {
-	    system qq{"$astyle" $cfile > NUL 2>&1};
-        } else {
-	    system qq{"$astyle" $cfile >/dev/null 2>&1};
-        }
+	    invoke_astyle($cfile) if defined ($astyle);
 	} elsif ($index == 2) {
-        if ($^O eq "MSWin32") {
-	    system qq{"$clang_format" -i $cfile > NUL 2>&1};
-        } else {
-	    system qq{"$clang_format" -i $cfile >/dev/null 2>&1};
-        }
+	    invoke_clang_format($cfile);
 	} else {
 	    return ($STOP, \$index);
 	}
     }
-    $index++;
+    my $new = read_file($cfile);
+    if ($old eq $new) {
+	$index++;
+	goto AGAIN;
+    }
+
     return ($OK, \$index);
 }
 
